﻿using System.Security.Cryptography;
using System.Text;

namespace DiplomaThesis.Core.Encryption;

/// <summary>
/// Encryptor using symmetric encryption
/// </summary>
public class SymmetricEncryptor
{
    /// <summary>
    /// Encrypts message with provided key
    /// </summary>
    /// <param name="encryptionKey">Key used for message encryption</param>
    /// <param name="message">Message to be encrypted</param>
    /// <returns>Tuple with encrypted message and initialization vector</returns>
    public (byte[] EncryptedMessage, byte[] IV) EncryptMessage(byte[] encryptionKey, string message)
    {
        // Convert message from string to bytes
        var encodedMessage = Encoding.UTF8.GetBytes(message);

        // Use AES instance
        using Aes aes = Aes.Create();

        // Set AES mode to CBC
        aes.Mode = CipherMode.CBC;

        // Set AES encryption key
        aes.Key = encryptionKey;

        // Generate initialization vector
        aes.GenerateIV();

        // Use AES encryptor instance
        using var cryptoTransform = aes.CreateEncryptor();

        // Encrypt message
        var encryptedMessage = cryptoTransform.TransformFinalBlock(encodedMessage, 0, encodedMessage.Length);

        // Return encrypted message and initialization vector
        return (encryptedMessage, aes.IV);
    }

    /// <summary>
    /// Decrypts encrypted message with provided key and initialization vector
    /// </summary>
    /// <param name="decryptionKey">Key to be used for decryption</param>
    /// <param name="iv">Initialization vector for AES</param>
    /// <param name="encryptedMessage">Encrypted message</param>
    /// <returns>Decrypted message</returns>
    public string DecryptMessage(byte[] decryptionKey, byte[] iv, byte[] encryptedMessage)
    {
        // Use AES instance
        using Aes aes = Aes.Create();

        // Set AES mode to CBC
        aes.Mode = CipherMode.CBC;

        // Set AES encryption key
        aes.Key = decryptionKey;

        // Set AES initialization vector
        aes.IV = iv;

        // Use AES decryptor instance
        using var cryptoTransform = aes.CreateDecryptor();

        // Decrypt message
        var decryptedMessage = cryptoTransform.TransformFinalBlock(encryptedMessage, 0, encryptedMessage.Length);

        // Return decrypted message
        return Encoding.UTF8.GetString(decryptedMessage);
    }
}
