﻿using DiplomaThesis.MobileApp.Helpers;
using DiplomaThesis.MobileApp.Models.Database;
using DiplomaThesis.MobileApp.Pages;
using DiplomaThesis.MobileApp.Services;
using DiplomaThesis.MobileApp.ViewModels.Base;
using LiteDB;
using System;
using System.Linq;
using System.Windows.Input;
using Xamarin.Forms;

namespace DiplomaThesis.MobileApp.ViewModels
{
    /// <summary>
    /// View model for add new chat page
    /// </summary>
    public class AddNewChatPageViewModel : BaseViewModel
    {
        /// <summary>
        /// Message service used for working with messages
        /// </summary>
        private readonly MessageService mMessageService = new MessageService();

        /// <summary>
        /// Server address
        /// </summary>
        public string Server { get; set; } = string.Empty;

        /// <summary>
        /// User id
        /// </summary>
        public string UserID { get; set; } = string.Empty;

        /// <summary>
        /// Command for starting new chat
        /// </summary>
        public ICommand StartChatCommand { get; set; }

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="addNewChatPage">Page to which is view model bind to</param>
        public AddNewChatPageViewModel(AddNewChatPage addNewChatPage)
        {
            StartChatCommand = new Command(async () =>
            {
                try
                {
                    using (var db = new LiteDatabase(LiteDbHelper.GetDatabasePath("AppDb.db")))
                    {
                        var account = db
                            .GetCollection<Account>("accounts")
                            .FindAll()
                            .FirstOrDefault();

                        var chat = await mMessageService.StartChat(
                            account.DisplayName,
                            account.Server,
                            account.UserId,
                            Server,
                            UserID);

                        db.GetCollection<Chat>("chats")
                            .Insert(chat);
                    }

                    await Shell.Current.GoToAsync("//chats");
                }
                catch (Exception ex)
                {
                    await addNewChatPage.DisplayAlert("Error", ex.Message, "Cancel");
                }
            });
        }
    }
}
