﻿using DiplomaThesis.MobileApp.Helpers;
using DiplomaThesis.MobileApp.Models.Database;
using DiplomaThesis.MobileApp.Pages;
using DiplomaThesis.MobileApp.Services;
using DiplomaThesis.MobileApp.ViewModels.Base;
using LiteDB;
using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Input;
using Xamarin.Forms;

namespace DiplomaThesis.MobileApp.ViewModels
{
    /// <summary>
    /// View model for chat page
    /// </summary>
    public class ChatPageViewModel : BaseViewModel
    {
        /// <summary>
        /// Message service used for working with messages
        /// </summary>
        private readonly MessageService mMessageService = new MessageService();

        /// <summary>
        /// Information about chat state
        /// </summary>
        public string ChatState => "Status: " + (Chat.IsActive ? "Active" : "Pending");

        /// <summary>
        /// Chat which is shown on page
        /// </summary>
        public Chat Chat { get; set; }

        /// <summary>
        /// Message which should be sent
        /// </summary>
        public string NewChatMessage { get; set; } = String.Empty;

        /// <summary>
        /// Command for sending the message
        /// </summary>
        public ICommand SendMessageCommand { get; set; }

        /// <summary>
        /// Messages
        /// </summary>
        public ObservableCollection<Message> Messages { get; set; } = new ObservableCollection<Message>();

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="chatPage">Page to which view model is bind to</param>
        public ChatPageViewModel(ChatPage chatPage, Chat chat)
        {
            Chat = chat;

            try
            {
                using (var db = new LiteDatabase(LiteDbHelper.GetDatabasePath("AppDb.db")))
                {
                    Messages = new ObservableCollection<Message>(db.GetCollection<Message>("messages")
                        .Find(x => x.ChatId == Chat.ChatId)
                        .ToList());
                }
            }
            catch (Exception ex)
            {
                chatPage.DisplayAlert("Error", ex.Message, "OK");
            }

            SendMessageCommand = new Command(async () =>
            {
                try
                {
                    if (!Chat.IsActive)
                    {
                        await chatPage.DisplayAlert("Error", "Chat is not yet active", "OK");
                        return;
                    }

                    await mMessageService.SendMessage(
                        Chat.PeerServer,
                        Chat.PeerUserId,
                        Chat.ChatId,
                        NewChatMessage,
                        Chat.EncryptionKey);

                    var newMessage = new Message()
                    {
                        ChatId = Chat.ChatId,
                        Date = DateTime.Now,
                        IsReceived = false,
                        Text = NewChatMessage
                    };

                    Messages.Add(newMessage);

                    using (var db = new LiteDatabase(LiteDbHelper.GetDatabasePath("AppDb.db")))
                    {
                        db.GetCollection<Message>("messages")
                            .Insert(newMessage);
                    }

                    NewChatMessage = string.Empty;
                }
                catch (Exception ex)
                {
                    await chatPage.DisplayAlert("Error", ex.Message, "OK");
                }
            });
        }
    }
}
