﻿using DiplomaThesis.MobileApp.Helpers;
using DiplomaThesis.MobileApp.Models.Database;
using DiplomaThesis.MobileApp.Pages;
using DiplomaThesis.MobileApp.Services;
using DiplomaThesis.MobileApp.ViewModels.Base;
using LiteDB;
using System;
using System.Windows.Input;
using Xamarin.Forms;

namespace DiplomaThesis.MobileApp.ViewModels
{
    /// <summary>
    /// View model for login page
    /// </summary>
    public class LoginPageViewModel : BaseViewModel
    {
        /// <summary>
        /// Auth service
        /// </summary>
        private readonly AuthService mAuthService = new AuthService();

        /// <summary>
        /// Server address
        /// </summary>
        public string Server { get; set; } = string.Empty;

        /// <summary>
        /// User id
        /// </summary>
        public string UserID { get; set; } = string.Empty;

        /// <summary>
        /// Key used for encrypted communication
        /// </summary>
        public string EncryptionKey { get; set; } = string.Empty;

        /// <summary>
        /// Command for navigating to register page
        /// </summary>
        public ICommand GoToRegisterPageCommand { get; }

        /// <summary>
        /// Command for sign in
        /// </summary>
        public ICommand LoginCommand { get; }

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="loginPage">Page to which view model is bind to</param>
        public LoginPageViewModel(LoginPage loginPage)
        {
            LoginCommand = new Command(async () =>
            {
                try
                {
                    var loginData = await mAuthService.Login(Server, UserID, EncryptionKey);

                    using (var db = new LiteDatabase(LiteDbHelper.GetDatabasePath("AppDb.db")))
                    {
                        var accounts = db.GetCollection<Account>("accounts");

                        accounts.Insert(new Account()
                        {
                            Server = Server,
                            EncryptionKey = EncryptionKey,
                            AccessToken = loginData.token,
                            IsLoggedIn = true,
                            UserId = UserID,
                            DisplayName = loginData.displayName
                        });
                    }
                }
                catch (Exception ex)
                {
                    await loginPage.DisplayAlert("Error", ex.Message, "Cancel");
                    return;
                }

                await Shell.Current.GoToAsync("//chats");
            });

            GoToRegisterPageCommand = new Command(async () =>
            {
                await Shell.Current.Navigation.PushAsync(new RegisterPage());
            });
        }
    }
}
