﻿using DiplomaThesis.MobileApp.Pages;
using DiplomaThesis.MobileApp.Services;
using DiplomaThesis.MobileApp.ViewModels.Base;
using System;
using System.Windows.Input;
using Xamarin.Essentials;
using Xamarin.Forms;

namespace DiplomaThesis.MobileApp.ViewModels
{
    /// <summary>
    /// View model for register page
    /// </summary>
    public class RegisterPageViewModel : BaseViewModel
    {
        /// <summary>
        /// Service used for user authentication
        /// </summary>
        private readonly AuthService mAuthService = new AuthService();

        /// <summary>
        /// User display name
        /// </summary>
        public string DisplayName { get; set; } = string.Empty;

        /// <summary>
        /// Server address
        /// </summary>
        public string Server { get; set; } = string.Empty;

        /// <summary>
        /// User id
        /// </summary>
        public string UserID { get; set; } = string.Empty;

        /// <summary>
        /// Key used for encrypted communication
        /// </summary>
        public string EncryptionKey { get; set; } = string.Empty;

        /// <summary>
        /// Command for copying server
        /// </summary>
        public ICommand CopyServerCommand { get; }

        /// <summary>
        /// Command for copying user id
        /// </summary>
        public ICommand CopyUserIdCommnad { get; }

        /// <summary>
        /// Command for copying encryption key
        /// </summary>
        public ICommand CopyEncryptionKeyCommand { get; }

        /// <summary>
        /// Command for registering new user
        /// </summary>
        public ICommand RegisterCommand { get; }

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="registerPage">Page to which view model is bind to</param>
        public RegisterPageViewModel(RegisterPage registerPage)
        {
            CopyServerCommand = new Command(async () =>
            {
                await Clipboard.SetTextAsync(Server);
            });

            CopyUserIdCommnad = new Command(async () =>
            {
                await Clipboard.SetTextAsync(UserID);
            });

            CopyEncryptionKeyCommand = new Command(async () =>
            {
                await Clipboard.SetTextAsync(EncryptionKey);
            });

            RegisterCommand = new Command(async () =>
            {
                try
                {
                    var registrationInfo = await mAuthService.Register(Server, DisplayName);

                    UserID = registrationInfo.UserId;
                    EncryptionKey = registrationInfo.EncryptionKey;
                }
                catch (Exception ex)
                {
                    await registerPage.DisplayAlert("Error", ex.Message, "Cancel");
                    return;
                }
            });
        }
    }
}
