﻿using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.DependencyInjection;

namespace DiplomaThesis.Server.Database;

/// <summary>
/// Helper class for database initialization
/// </summary>
public class DatabaseInitializer
{
    /// <summary>
    /// Initializes application database
    /// </summary>
    /// <param name="app">Application which will use database</param>
    public async void InitializeDb(WebApplication app)
    {
        // Get new scope
        using var scope = app.Services.CreateScope();

        // Get database context from scope
        using var context = scope.ServiceProvider.GetService<ApplicationDbContext>();
        
        // Get database from database context
        var database = context.Database;

        // Make sure database is created
        await database.EnsureCreatedAsync()!;
    }
}
