﻿using DiplomaThesis.Core.Models.Inbox;
using System;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace DiplomaThesis.Server.Models;

/// <summary>
/// Record representing inbox message
/// </summary>
public record InboxMessageModel
{
    /// <summary>
    /// Content of the message
    /// </summary>
    [Required]
    public string Content { get; init; }

    /// <summary>
    /// Id of the chat to which message belongs to
    /// </summary>
    [Required]
    public string ChatId { get; init; }

    /// <summary>
    /// Id of the inbox message
    /// </summary>
    [DatabaseGenerated(DatabaseGeneratedOption.Identity)]
    [Key]
    [Required]
    public Guid Id { get; init; }

    /// <summary>
    /// Initialization vector for encryption
    /// </summary>
    [Required]
    public string IV { get; init; }

    /// <summary>
    /// Type of the inbox message
    /// </summary>
    [Required]
    public InboxMessageType MessageType { get; init; }

    /// <summary>
    /// Receiver of the message
    /// </summary>
    [Required]
    public UserModel Receiver { get; init; }
}
