﻿using System;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace DiplomaThesis.Server.Models;

/// <summary>
/// Record representing user model
/// </summary>
public record UserModel
{
    /// <summary>
    /// Access token of the user
    /// </summary>
    [Required]
    public AccessTokenModel AccessToken { get; init; }
    
    /// <summary>
    /// Authentication information of user
    /// </summary>
    [Required]
    public AuthInfoModel AuthInfo { get; init; }

    /// <summary>
    /// User display name
    /// </summary>
    [Required]
    [MaxLength(256)]
    public string DisplayName { get; init; }

    /// <summary>
    /// Id of the user
    /// </summary>
    [DatabaseGenerated(DatabaseGeneratedOption.Identity)]
    [Key]
    [Required]
    public Guid Id { get; init; }
}
