using DiplomaThesis.Core.Encryption;
using DiplomaThesis.Server.Database;
using DiplomaThesis.Server.Routes;
using Microsoft.AspNetCore.Builder;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;

// Create application builder
var builder = WebApplication.CreateBuilder(args);

// Add services to application
builder.Services
    .AddDbContext<ApplicationDbContext>(options =>
    {
        // Set the database connection string
        options.UseNpgsql(builder.Configuration["ConnectionStrings:DatabaseConnection"]);
    })
    .AddSingleton<BigIntegerGenerator>()
    .AddSingleton<SymmetricEncryptor>()
    .AddSingleton<DiffieHellman>();

// Build application
var app = builder.Build();

// Show error page in developer mode
if (app.Environment.IsDevelopment())
{
    app.UseDeveloperExceptionPage();
}

// Initialize application database
new DatabaseInitializer().InitializeDb(app);

// Map routes
app.MapInboxRoute()
    .MapTokenRoute()
    .MapUserRoute();

// Run application
app.Run();
